awk '
BEGIN {
  print "<!DOCTYPE html PUBLIC \"-//W3C//DTD HTML 4.0 Transitional//EN\" " \
        "\"http://www.w3.org/TR/REC-html40/loose.dtd\">"
  print "<html><head>"
  print "<style type=\"text/css\">"
  print "TT { font-size: larger }"
  print "H4 { text-decoration: underline }"
  print "TABLE { cellpadding: 2 }"
  print "TH { background-color: #E1E1E1 }"
  print "TD { background-color: #F1F1F1 }"
  print "TR.silver { background-color: silver }"
  print "</style>"
  print "<title>SIB-PAIR: List of subroutines and functions</title></head>"
  print "<body>"
  print "<H1>SIB-PAIR: List of subroutines and functions</H1>"
  print "<p>"
  print "Sib-pair is largely written in standard Fortran 95 (as usual, there"
  print "are a few system specific routines for file handling, time and date etc). "
  print "<p>"
  print "The Sib-pair source code is distributed as a single file, with"
  print "C-preprocesser macros to deal with platform specific code."
  print "The \"source lines\" in the tables below are the lines within that single file."
  print "<p>"
  print "The listings below are automatically generated"
  print "(by <a href=\"autodoc.sh\">an awk script</a>), so check the"
  print "<a href=\"./sib-pair.f95\">source</a>"
  print "for additional informative comments that may have been skipped."
  print "<p>"
  print "Listing is first by position in the source code file, then alphabetically."
  print "The organization of the source is modules, main, I/O, statistical"
  print "library routines, association and linkage."
  printf "\n"
}
substr($1,1,1) ~ "!" {
  comment=substr($0,2)
  getline
  while (substr($1,1,1) ~ "!") {
    comment=comment "\n" substr($0,2)
    getline
  }
}
$1!="subroutine" && $1!="module" && $1!="function" {
  comment=""
}
$1=="interface" {
  do {
    getline
  } while ($1!="end" || $2!="interface")
}
$1=="module" {
  module=$2
  modcomment=comment
}
$1=="end" && $2=="module" {
  module=""
}
$1=="subroutine" || $1=="function" {
  gsub("^[ ]+","") 
  name=substr($2,1,index($2,"(")-1)
  nroutine++
  routine[nroutine]=name
  typ=$1
  gsub("subroutine","")
  gsub("function","")
  sta=NR
  if ($0 !~ "&[ ]*$") {
    call=$0 
  }else{
    call=substr($0, 1,  match($0,"&[ ]*$")-1)
    while ($0 ~ "&[ ]*$") {
      getline 
      gsub("^[ ]+","") 
      if ($0 !~ "&[ ]*$") {
        call=call $0 
      }else{
        call=call substr($0, 1, match($0,"&[ ]*$")-1)
      }
    }
  }
  do {
    getline
    if ($1=="interface") {
      do {
        getline
      } while ($1!="end" || $2!="interface")
    }
  } while ($1!="end" || $2!=typ)
  fin=NR
  if (module=="") {
    print "<H2><A name=\"" name "\">" name "</a><H2>"
  }else{
    print "<H2><A name=\"" name "\">" name " {<i>" module "</i>}</a><H2>"
  }
  print "<table>"
  print "<tr><th>Name of routine:<td><b>",name,"</b>"
  print "<tr><th>Source lines:<td>", sta " - " fin
  print "<tr><th>Type<td>", typ
  printf "<tr><th>Call<td>" call
  printf "<tr><th>Module"
  if (module != "") {
    print "<td>" module
    if (comment=="") {
      gsub("&","\\&amp;",modcomment)
      gsub(">","\\&gt;",modcomment)
      gsub("<","\\&lt;",modcomment)
      print "<tr><th>Notes<td><pre>" modcomment "</pre>"
    }
  }else{
    print "<td>None"
  }
  if (comment!="") {
    gsub("&","\\&amp;",comment)
    gsub(">","\\&gt;",comment)
    gsub("<","\\&lt;",comment)
    print "<tr><th>Notes<td><pre>" comment "\n</pre>"
  }
  print "</table><p>"
}
END { print "<HR>"
      print "<H2>Alphabetical list of routines</H2>"
      print "<p>"
      print "<table>"
      nroutine=asort(routine)
      for(i=1;i<=nroutine;i++) {
        if (i%5==1) print "<tr>"
        print "<td><a href=\"#" routine[i] "\">" routine[i] "</a>"
      }
      print "</table>"
      print "</body></html>"
}' $0

